@@ -1,7 +1,7 @@
-# Copyright (c) 2007-2008 Markus Reder and Roger Bivand
+# Copyright (c) 2007-2022 Markus Reder and Roger Bivand
 
 localmoran.exact.alt <- function(model, select, nb, glist = NULL, style = "W",
-    zero.policy = NULL, alternative = "greater", spChk=NULL, 
+    zero.policy = NULL, alternative = "two.sided", spChk=NULL, 
     resfun=weighted.residuals, Omega=NULL, save.Vi = FALSE, save.M=FALSE,
     useTP=FALSE, truncErr=1e-6, zeroTreat=0.1) {
 # need to impose check on weights TODO!!
@@ -34,7 +34,7 @@ localmoran.exact.alt <- function(model, select, nb, glist = NULL, style = "W",
     u <- as.vector(u)
     select <- unique(as.integer(select))
     if (length(select) < 1L) stop("select too short")
-    if (any(select < 1 || select > n))
+    if (any(select < 1) || any(select > n))
         stop("select out of range")
     utu <- c(crossprod(u))
     p <- model$rank
@@ -63,8 +63,25 @@ localmoran.exact.alt <- function(model, select, nb, glist = NULL, style = "W",
 # correction by Danlin Yu, 25 March 2004
 	a <- sum(sapply(B$weights, function(x) sqrt(sum(x^2))))
     } else if (style == "C") a <- sum(unlist(B$weights))
-    res <- vector(mode="list", length=length(select))
-    for (i in 1:length(select)) {
+
+    cores <- get.coresOption()
+    if (is.null(cores)) {
+        parallel <- "no"
+    } else {
+        parallel <- ifelse (get.mcOption(), "multicore", "snow")
+    }
+    ncpus <- ifelse(is.null(cores), 1L, cores)
+    cl <- NULL
+    if (parallel == "snow") {
+        cl <- get.ClusterOption()
+        if (is.null(cl)) {
+            parallel <- "no"
+            warning("no cluster in ClusterOption, parallel set to no")
+        }
+    }
+
+    exactLocalMoranAlt_int <- function(i, B, select, style, n, D, a, 
+        zero.policy, u, utu, M1, M2, useTP, truncErr, zeroTreat, alternative) {
         Vi <- listw2star(B, select[i], style=style, n, D, a,
 	    zero.policy=zero.policy)
         Viu <- lag.listw(Vi, u, zero.policy=TRUE)
@@ -73,6 +90,7 @@ localmoran.exact.alt <- function(model, select, nb, glist = NULL, style = "W",
         obj <- exactLocalMoranAlt(Ii=Ii, Vi=Vi, M1=M1, M2=M2, n=n,
             alternative=alternative, useTP=useTP, truncErr=truncErr,
                 zeroTreat=zeroTreat)
+
         data.name <- paste("region:", select[i],
 	    attr(nb, "region.id")[select[i]],
 	    "\n", paste(strwrap(paste("model: ", gsub("[ ]+", " ", 
@@ -84,8 +102,56 @@ localmoran.exact.alt <- function(model, select, nb, glist = NULL, style = "W",
         obj$df <- (n-p)
         obj$i <- paste(select[i], attr(nb, "region.id")[select[i]])
         obj$Vi <- if(save.Vi) Vi else NULL
-	res[[i]] <- obj
+	obj
     }
+    if (parallel == "snow") {
+      if (requireNamespace("parallel", quietly = TRUE)) {
+        sI <- parallel::splitIndices(n, length(cl))
+        env <- new.env()
+        assign("select", select, envir=env)
+        assign("B", B, envir=env)
+        assign("style", style, envir=env)
+        assign("n", n, envir=env)
+        assign("D", D, envir=env)
+        assign("a", a, envir=env)
+        assign("zero.policy", zero.policy, envir=env)
+        assign("alternative", alternative, envir=env)
+        assign("u", u, envir=env)
+        assign("utu", utu, envir=env)
+        assign("M1", M2, envir=env)
+        assign("M2", M2, envir=env)
+        assign("useTP", useTP, envir=env)
+        assign("truncErr", truncErr, envir=env)
+        assign("zeroTreat", zeroTreat, envir=env)
+        parallel::clusterExport(cl, varlist=c("select", "B", "style",
+            "n", "D", "a", "zero.policy", "alternative", "u", "M1", "M2",
+            "utu", "useTP", "truncErr", "zeroTreat"), envir=env)
+        oo <- parallel::clusterApply(cl, x = sI, fun=lapply, function(i) {
+            exactLocalMoranAlt_int(i, B, select, style, n, D, a, 
+            zero.policy, u, utu, M1, M2, useTP, truncErr, zeroTreat,
+            alternative)})
+        res <- do.call("c", oo)
+        rm(env)
+      } else {
+        stop("parallel not available")
+      }
+    } else if (parallel == "multicore") {
+      if (requireNamespace("parallel", quietly = TRUE)) {
+        sI <- parallel::splitIndices(n, ncpus)
+        oo <- parallel::mclapply(sI, FUN=lapply, function(i) {
+            exactLocalMoranAlt_int(i, B, select, style, n, D, a, 
+            zero.policy, u, utu, M1, M2, useTP, truncErr, zeroTreat,
+            alternative)}, mc.cores=ncpus)
+        res <- do.call("c", oo)
+      } else {
+        stop("parallel not available")
+      }
+    } else {
+        res <- lapply(1:n, function(i) exactLocalMoranAlt_int(i, B, select, 
+            style, n, D, a, zero.policy, u, utu, M1, M2, useTP, truncErr, 
+            zeroTreat, alternative))
+    }
+
     class(res) <- "localmoranex"
     if (save.M) attr(res, "M") <- list(M1=M1, M2=M2)
     res
